@@ -4,7 +4,7 @@
 Discord API Wrapper
 ~~~~~~~~~~~~~~~~~~~
 
-A basic wrapper for the Discord API.
+A basic wrapper for the Discord user API.
 
 :copyright: (c) 2015-present Rapptz
 :license: MIT, see LICENSE for more details.
@@ -15,7 +15,7 @@ __title__ = 'discord'
 __author__ = 'Rapptz'
 __license__ = 'MIT'
 __copyright__ = 'Copyright 2015-present Rapptz'
-__version__ = '1.8.1'
+__version__ = '1.9.1'
 
 __path__ = __import__('pkgutil').extend_path(__path__, __name__)
 
@@ -51,7 +51,6 @@ from . import utils, opus, abc
 from .enums import *
 from .embeds import Embed
 from .mentions import AllowedMentions
-from .shard import AutoShardedClient, ShardInfo
 from .player import *
 from .webhook import *
 from .voice_client import VoiceClient, VoiceProtocol
@@ -62,6 +61,6 @@ from .sticker import Sticker
 
 VersionInfo = namedtuple('VersionInfo', 'major minor micro releaselevel serial')
 
-version_info = VersionInfo(major=1, minor=8, micro=1, releaselevel='final', serial=0)
+version_info = VersionInfo(major=1, minor=9, micro=1, releaselevel='final', serial=0)
 
 logging.getLogger(__name__).addHandler(logging.NullHandler())
