@@ -55,8 +55,7 @@ __all__ = (
 
 class ReconnectWebSocket(Exception):
     """Signals to safely reconnect the websocket."""
-    def __init__(self, shard_id, *, resume=True):
-        self.shard_id = shard_id
+    def __init__(self, *, resume=True):
         self.resume = resume
         self.op = 'RESUME' if resume else 'IDENTIFY'
 
@@ -74,7 +73,6 @@ class GatewayRatelimiter:
         self.window = 0.0
         self.per = per
         self.lock = asyncio.Lock()
-        self.shard_id = None
 
     def is_ratelimited(self):
         current = time.time()
@@ -104,7 +102,7 @@ class GatewayRatelimiter:
         async with self.lock:
             delta = self.get_delay()
             if delta:
-                log.warning('WebSocket in shard ID %s is ratelimited, waiting %.2f seconds', self.shard_id, delta)
+                log.warning('WebSocket is ratelimited, waiting %.2f seconds', delta)
                 await asyncio.sleep(delta)
 
 
@@ -112,16 +110,14 @@ class KeepAliveHandler(threading.Thread):
     def __init__(self, *args, **kwargs):
         ws = kwargs.pop('ws', None)
         interval = kwargs.pop('interval', None)
-        shard_id = kwargs.pop('shard_id', None)
         threading.Thread.__init__(self, *args, **kwargs)
         self.ws = ws
         self._main_thread_id = ws.thread_id
         self.interval = interval
         self.daemon = True
-        self.shard_id = shard_id
-        self.msg = 'Keeping shard ID %s websocket alive with sequence %s.'
-        self.block_msg = 'Shard ID %s heartbeat blocked for more than %s seconds.'
-        self.behind_msg = 'Can\'t keep up, shard ID %s websocket is %.1fs behind.'
+        self.msg = 'Keeping websocket alive with sequence %s.'
+        self.block_msg = 'Heartbeat blocked for more than %s seconds.'
+        self.behind_msg = 'Can\'t keep up, websocket is %.1fs behind.'
         self._stop_ev = threading.Event()
         self._last_ack = time.perf_counter()
         self._last_send = time.perf_counter()
@@ -132,7 +128,7 @@ class KeepAliveHandler(threading.Thread):
     def run(self):
         while not self._stop_ev.wait(self.interval):
             if self._last_recv + self.heartbeat_timeout < time.perf_counter():
-                log.warning("Shard ID %s has stopped responding to the gateway. Closing and restarting.", self.shard_id)
+                log.warning("Bot has stopped responding to the gateway. Closing and restarting.")
                 coro = self.ws.close(4000)
                 f = asyncio.run_coroutine_threadsafe(coro, loop=self.ws.loop)
 
@@ -145,7 +141,7 @@ class KeepAliveHandler(threading.Thread):
                     return
 
             data = self.get_payload()
-            log.debug(self.msg, self.shard_id, data['d'])
+            log.debug(self.msg, data['d'])
             coro = self.ws.send_heartbeat(data)
             f = asyncio.run_coroutine_threadsafe(coro, loop=self.ws.loop)
             try:
@@ -164,7 +160,7 @@ class KeepAliveHandler(threading.Thread):
                         else:
                             stack = traceback.format_stack(frame)
                             msg = '%s\nLoop thread traceback (most recent call last):\n%s' % (self.block_msg, ''.join(stack))
-                        log.warning(msg, self.shard_id, total)
+                        log.warning(msg, total)
 
             except Exception:
                 self.stop()
@@ -188,15 +184,15 @@ class KeepAliveHandler(threading.Thread):
         self._last_ack = ack_time
         self.latency = ack_time - self._last_send
         if self.latency > 10:
-            log.warning(self.behind_msg, self.shard_id, self.latency)
+            log.warning(self.behind_msg, self.latency)
 
 class VoiceKeepAliveHandler(KeepAliveHandler):
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
         self.recent_ack_latencies = deque(maxlen=20)
-        self.msg = 'Keeping shard ID %s voice websocket alive with timestamp %s.'
-        self.block_msg = 'Shard ID %s voice heartbeat blocked for more than %s seconds'
-        self.behind_msg = 'High socket latency, shard ID %s heartbeat is %.1fs behind'
+        self.msg = 'Keeping voice websocket alive with timestamp %s.'
+        self.block_msg = 'Voice heartbeat blocked for more than %s seconds'
+        self.behind_msg = 'High socket latency, heartbeat is %.1fs behind'
 
     def get_payload(self):
         return {
@@ -248,7 +244,11 @@ class DiscordWebSocket:
         Receive only. Confirms receiving of a heartbeat. Not having it implies
         a connection issue.
     GUILD_SYNC
-        Send only. Requests a guild sync.
+        Send only. Requests a guild sync. This is unfortunately no longer functional.
+    ACCESS_DM
+        Send only. Useless.
+    GUILD_SUBSCRIBE
+        Send only. Subscribes you to guilds. Might respond with GUILD_MEMBER_LIST_UPDATE.
     gateway
         The gateway we are currently connected to.
     token
@@ -268,6 +268,8 @@ class DiscordWebSocket:
     HELLO              = 10
     HEARTBEAT_ACK      = 11
     GUILD_SYNC         = 12
+    ACCESS_DM          = 13
+    GUILD_SUBSCRIBE    = 14
 
     def __init__(self, socket, *, loop):
         self.socket = socket
@@ -297,7 +299,7 @@ class DiscordWebSocket:
         return self._rate_limiter.is_ratelimited()
 
     @classmethod
-    async def from_client(cls, client, *, initial=False, gateway=None, shard_id=None, session=None, sequence=None, resume=False):
+    async def from_client(cls, client, *, initial=False, gateway=None, session=None, sequence=None, resume=False):
         """Creates a main websocket for Discord from a :class:`Client`.
 
         This is for internal use only.
@@ -314,12 +316,12 @@ class DiscordWebSocket:
         ws.gateway = gateway
         ws.call_hooks = client._connection.call_hooks
         ws._initial_identify = initial
-        ws.shard_id = shard_id
-        ws._rate_limiter.shard_id = shard_id
-        ws.shard_count = client._connection.shard_count
         ws.session_id = session
         ws.sequence = sequence
         ws._max_heartbeat_timeout = client._connection.heartbeat_timeout
+        ws._user_agent = client.http.user_agent
+        ws._super_properties = client.http.super_properties
+        ws._zlib_enabled = client.http.zlib
 
         client._connection._update_references(ws)
 
@@ -363,40 +365,33 @@ class DiscordWebSocket:
     async def identify(self):
         """Sends the IDENTIFY packet."""
         payload = {
-             'op': self.IDENTIFY,
-             'd': {
-                 'token': self.token,
-                 'capabilities': 61,
-                 'properties': {
-                     '$os': 'Windows',
-                     '$browser': 'Chrome',
-                     '$referrer': '',
-                     '$referring_domain': ''
-                 },
-                 'compress': True,
-                 'large_threshold': 250,
-                 'guild_subscriptions': self._connection.guild_subscriptions,
-             }
-         }
-
-        if not self._connection.is_bot:
-            payload['d']['synced_guilds'] = []
-
-        if self.shard_id is not None and self.shard_count is not None:
-            payload['d']['shard'] = [self.shard_id, self.shard_count]
-
-        state = self._connection
-        if state._activity is not None or state._status is not None:
-            payload['d']['presence'] = {
-                'status': state._status,
-                'game': state._activity,
-                'since': 0,
-                'afk': False
+            'op': self.IDENTIFY,
+            'd': {
+                'token': self.token,
+                'capabilities': 125,
+                'properties': self._super_properties,
+                'presence': {
+                    'status': 'online',
+                    'since': 0,
+                    'activities': [],
+                    'afk': False
+                },
+                'compress': False,
+                'client_state': {
+                    'guild_hashes': {},
+                    'highest_last_message_id': '0',
+                    'read_state_version': 0,
+                    'user_guild_settings_version': -1
+                }
             }
+        }
 
-        await self.call_hooks('before_identify', self.shard_id, initial=self._initial_identify)
+        if not self._zlib_enabled:
+            payload['d']['compress'] = True
+
+        await self.call_hooks('before_identify', initial=self._initial_identify)
         await self.send_as_json(payload)
-        log.info('Shard ID %s has sent the IDENTIFY payload.', self.shard_id)
+        log.info('Bot has sent the IDENTIFY payload.')
 
     async def resume(self):
         """Sends the RESUME packet."""
@@ -410,7 +405,7 @@ class DiscordWebSocket:
         }
 
         await self.send_as_json(payload)
-        log.info('Shard ID %s has sent the RESUME payload.', self.shard_id)
+        log.info('Bot has sent the RESUME payload.')
 
     async def received_message(self, msg):
         self._dispatch('socket_raw_receive', msg)
@@ -425,7 +420,7 @@ class DiscordWebSocket:
             self._buffer = bytearray()
         msg = json.loads(msg)
 
-        log.debug('For Shard ID %s: WebSocket Event: %s', self.shard_id, msg)
+        log.debug('WebSocket Event: %s', msg)
         self._dispatch('socket_response', msg)
 
         op = msg.get('op')
@@ -444,7 +439,7 @@ class DiscordWebSocket:
                 # internal exception signalling to reconnect.
                 log.debug('Received RECONNECT opcode.')
                 await self.close()
-                raise ReconnectWebSocket(self.shard_id)
+                raise ReconnectWebSocket()
 
             if op == self.HEARTBEAT_ACK:
                 if self._keep_alive:
@@ -459,7 +454,7 @@ class DiscordWebSocket:
 
             if op == self.HELLO:
                 interval = data['heartbeat_interval'] / 1000.0
-                self._keep_alive = KeepAliveHandler(ws=self, interval=interval, shard_id=self.shard_id)
+                self._keep_alive = KeepAliveHandler(ws=self, interval=interval)
                 # send a heartbeat immediately
                 await self.send_as_json(self._keep_alive.get_payload())
                 self._keep_alive.start()
@@ -468,13 +463,13 @@ class DiscordWebSocket:
             if op == self.INVALIDATE_SESSION:
                 if data is True:
                     await self.close()
-                    raise ReconnectWebSocket(self.shard_id)
+                    raise ReconnectWebSocket()
 
                 self.sequence = None
                 self.session_id = None
-                log.info('Shard ID %s session has been invalidated.', self.shard_id)
+                log.info('Session has been invalidated.')
                 await self.close(code=1000)
-                raise ReconnectWebSocket(self.shard_id, resume=False)
+                raise ReconnectWebSocket(resume=False)
 
             log.warning('Unknown OP code %s.', op)
             return
@@ -485,23 +480,20 @@ class DiscordWebSocket:
             self._trace = trace = data.get('_trace', [])
             self.sequence = msg['s']
             self.session_id = data['session_id']
-            # pass back shard ID to ready handler
-            data['__shard_id__'] = self.shard_id
-            log.info('Shard ID %s has connected to Gateway: %s (Session ID: %s).',
-                     self.shard_id, ', '.join(trace), self.session_id)
+            log.info('Bot has connected to Gateway: %s (Session ID: %s).',
+                     ', '.join(trace), self.session_id)
 
         elif event == 'RESUMED':
             self._trace = trace = data.get('_trace', [])
-            # pass back the shard ID to the resumed handler
-            data['__shard_id__'] = self.shard_id
-            log.info('Shard ID %s has successfully RESUMED session %s under trace %s.',
-                     self.shard_id, self.session_id, ', '.join(trace))
+            log.info('Bot has successfully RESUMED session %s under trace %s.',
+                     self.session_id, ', '.join(trace))
 
         try:
             func = self._discord_parsers[event]
         except KeyError:
             log.debug('Unknown event %s.', event)
         else:
+            log.info('Parsing event %s.', event)
             func(data)
 
         # remove the dispatched listeners
@@ -567,15 +559,15 @@ class DiscordWebSocket:
 
             if isinstance(e, asyncio.TimeoutError):
                 log.info('Timed out receiving packet. Attempting a reconnect.')
-                raise ReconnectWebSocket(self.shard_id) from None
+                raise ReconnectWebSocket() from None
 
             code = self._close_code or self.socket.close_code
             if self._can_handle_close():
                 log.info('Websocket closed with %s, attempting a reconnect.', code)
-                raise ReconnectWebSocket(self.shard_id) from None
+                raise ReconnectWebSocket() from None
             else:
                 log.info('Websocket closed with %s, cannot reconnect.', code)
-                raise ConnectionClosed(self.socket, shard_id=self.shard_id, code=code) from None
+                raise ConnectionClosed(self.socket, code=code) from None
 
     async def send(self, data):
         await self._rate_limiter.block()
@@ -587,7 +579,7 @@ class DiscordWebSocket:
             await self.send(utils.to_json(data))
         except RuntimeError as exc:
             if not self._can_handle_close():
-                raise ConnectionClosed(self.socket, shard_id=self.shard_id) from exc
+                raise ConnectionClosed(self.socket) from exc
 
     async def send_heartbeat(self, data):
         # This bypasses the rate limit handling code since it has a higher priority
@@ -595,7 +587,7 @@ class DiscordWebSocket:
             await self.socket.send_str(utils.to_json(data))
         except RuntimeError as exc:
             if not self._can_handle_close():
-                raise ConnectionClosed(self.socket, shard_id=self.shard_id) from exc
+                raise ConnectionClosed(self.socket) from exc
 
     async def change_presence(self, *, activity=None, status=None, afk=False, since=0.0):
         if activity is not None:
@@ -620,36 +612,48 @@ class DiscordWebSocket:
         log.debug('Sending "%s" to change status', sent)
         await self.send(sent)
 
-    async def request_sync(self, guild_ids):
+    async def request_lazy_guild(self, guild_id, *, typing=None, threads=None, activities=None, members=None, channels=None, thread_member_lists=None):
         payload = {
-            'op': self.GUILD_SYNC,
-            'd': list(guild_ids)
+            'op': self.GUILD_SUBSCRIBE,
+            'd': {
+                'guild_id': guild_id,
+            }
         }
+
+        data = payload['d']
+        if typing is not None:
+            data['typing'] = typing
+        if threads is not None:
+            data['threads'] = threads
+        if activities is not None:
+            data['activities'] = activities
+        if members is not None:
+            data['members'] = members
+        if channels is not None:
+            data['channels'] = channels
+        if thread_member_lists is not None:
+            data['thread_member_lists'] = thread_member_lists
+
         await self.send_as_json(payload)
 
-    async def request_chunks(self, guild_id, query=None, *, limit, user_ids=None, presences=False, nonce=None):
+    async def request_chunks(self, guild_ids, query=None, *, limit=None, user_ids=None, presences=True, nonce=None):
         payload = {
             'op': self.REQUEST_MEMBERS,
             'd': {
-                'guild_id': guild_id,
+                'guild_id': guild_ids,
+                'query': query,
+                'limit': limit,
                 'presences': presences,
-                'limit': limit
+                'user_ids': user_ids,
             }
         }
 
         if nonce:
             payload['d']['nonce'] = nonce
 
-        if user_ids:
-            payload['d']['user_ids'] = user_ids
-
-        if query is not None:
-            payload['d']['query'] = query
-
-
         await self.send_as_json(payload)
 
-    async def voice_state(self, guild_id, channel_id, self_mute=False, self_deaf=False):
+    async def voice_state(self, guild_id=None, channel_id=None, self_mute=False, self_deaf=False, self_video=None, self_stream=None, *, preferred_region=None):
         payload = {
             'op': self.VOICE_STATE,
             'd': {
@@ -660,9 +664,27 @@ class DiscordWebSocket:
             }
         }
 
+        if self_video is not None:
+            payload['d']['self_video'] = self_video
+        if self_stream is not None:
+            payload['d']['self_stream'] = self_stream
+        if preferred_region is not None:
+            payload['d']['preferred_region'] = preferred_region
+
         log.debug('Updating our voice state to %s.', payload)
         await self.send_as_json(payload)
 
+    async def access_dm(self, channel_id):
+        payload = {
+            'op': self.ACCESS_DM,
+            'd': {
+                'channel_id': channel_id
+            }
+        }
+
+        log.debug('Sending ACCESS_DM for channel %s.', channel_id)
+        await self.send_as_json(payload)
+
     async def close(self, code=4000):
         if self._keep_alive:
             self._keep_alive.stop()
@@ -758,7 +780,7 @@ class DiscordVoiceWebSocket:
         """Creates a voice websocket for the :class:`VoiceClient`."""
         gateway = 'wss://' + client.endpoint + '/?v=4'
         http = client._state.http
-        socket = await http.ws_connect(gateway, compress=15)
+        socket = await http.ws_connect(gateway, compress=15, host=client.endpoint)
         ws = cls(socket, loop=client.loop)
         ws.gateway = gateway
         ws._connection = client
@@ -885,10 +907,10 @@ class DiscordVoiceWebSocket:
             await self.received_message(json.loads(msg.data))
         elif msg.type is aiohttp.WSMsgType.ERROR:
             log.debug('Received %s', msg)
-            raise ConnectionClosed(self.ws, shard_id=None) from msg.data
+            raise ConnectionClosed(self.ws) from msg.data
         elif msg.type in (aiohttp.WSMsgType.CLOSED, aiohttp.WSMsgType.CLOSE, aiohttp.WSMsgType.CLOSING):
             log.debug('Received %s', msg)
-            raise ConnectionClosed(self.ws, shard_id=None, code=self._close_code)
+            raise ConnectionClosed(self.ws, code=self._close_code)
 
     async def close(self, code=1000):
         if self._keep_alive is not None:
