@@ -33,13 +33,13 @@ from operator import attrgetter
 import discord.abc
 
 from . import utils
-from .errors import ClientException
 from .user import BaseUser, User
 from .activity import create_activity
 from .permissions import Permissions
-from .enums import Status, try_enum
+from .enums import RelationshipAction, Status, try_enum
 from .colour import Colour
 from .object import Object
+from .asset import Asset
 
 class VoiceState:
     """Represents a Discord user's voice state.
@@ -206,17 +206,16 @@ class Member(discord.abc.Messageable, _BaseUser):
     premium_since: Optional[:class:`datetime.datetime`]
         A datetime object that specifies the date and time in UTC when the member used their
         Nitro boost on the guild, if available. This could be ``None``.
+    guild_avatar: Optional[:class:`str`]
+        The guild avatar hash the member has. Could be None.
     """
 
     __slots__ = ('_roles', 'joined_at', 'premium_since', '_client_status',
-                 'activities', 'guild', 'pending', 'nick', '_user', '_state')
+                 'activities', 'guild', 'pending', 'nick', '_user', '_state', 'guild_avatar')
 
     def __init__(self, *, data, guild, state):
         self._state = state
-        if 'user' in data:
-            self._user = state.store_user(data['user'])
-        else:
-            self._user = state.store_lazy_user(data['user_id'])
+        self._user = state.store_user(data['user'])
         self.guild = guild
         self.joined_at = utils.parse_time(data.get('joined_at'))
         self.premium_since = utils.parse_time(data.get('premium_since'))
@@ -227,6 +226,7 @@ class Member(discord.abc.Messageable, _BaseUser):
         self.activities = tuple(map(create_activity, data.get('activities', [])))
         self.nick = data.get('nick', None)
         self.pending = data.get('pending', False)
+        self.guild_avatar = data.get('avatar')
 
     def __str__(self):
         return str(self._user)
@@ -256,6 +256,7 @@ class Member(discord.abc.Messageable, _BaseUser):
         self._update_roles(data)
         self.nick = data.get('nick', None)
         self.pending = data.get('pending', False)
+        self.guild_avatar = data.get('avatar')
 
     @classmethod
     def _try_upgrade(cls, *,  data, guild, state):
@@ -292,6 +293,7 @@ class Member(discord.abc.Messageable, _BaseUser):
         self.pending = member.pending
         self.activities = member.activities
         self._state = member._state
+        self.guild_avatar = member.guild_avatar
 
         # Reference will not be copied unless necessary by PRESENCE_UPDATE
         # See below
@@ -318,6 +320,11 @@ class Member(discord.abc.Messageable, _BaseUser):
         except KeyError:
             pass
 
+        try:
+            self.guild_avatar = data['avatar']
+        except KeyError:
+            pass
+
         self.premium_since = utils.parse_time(data.get('premium_since'))
         self._update_roles(data)
 
@@ -549,6 +556,58 @@ class Member(discord.abc.Messageable, _BaseUser):
         """Optional[:class:`VoiceState`]: Returns the member's current voice state."""
         return self.guild._voice_state_for(self._user.id)
 
+    @property
+    def guild_avatar_url(self):
+        """:class:`Asset`: Returns an :class:`Asset` for the guild avatar the user has.
+
+        If the user does not have a guild avatar, an asset for
+        the user avatar is returned instead.
+
+        This is equivalent to calling :meth:`guild_avatar_url_as` with
+        the default parameters (i.e. webp/gif detection and a size of 1024).
+        """
+        return self.guild_avatar_url_as(format=None, size=1024)
+
+    def is_guild_avatar_animated(self):
+        """:class:`bool`: Indicates if the user has an animated guild avatar."""
+        return bool(self.guild_avatar and self.guild_avatar.startswith('a_'))
+
+    def guild_avatar_url_as(self, *, format=None, static_format='webp', size=1024):
+        """Returns an :class:`Asset` for the guild avatar the user has.
+
+        If the user does not have a guild avatar, an asset for
+        the user avatar is returned instead.
+
+        The format must be one of 'webp', 'jpeg', 'jpg', 'png' or 'gif', and
+        'gif' is only valid for animated avatars. The size must be a power of 2
+        between 16 and 4096.
+
+        Parameters
+        -----------
+        format: Optional[:class:`str`]
+            The format to attempt to convert the avatar to.
+            If the format is ``None``, then it is automatically
+            detected into either 'gif' or static_format depending on the
+            avatar being animated or not.
+        static_format: Optional[:class:`str`]
+            Format to attempt to convert only non-animated avatars to.
+            Defaults to 'webp'
+        size: :class:`int`
+            The size of the image to display.
+
+        Raises
+        ------
+        InvalidArgument
+            Bad image format passed to ``format`` or ``static_format``, or
+            invalid ``size``.
+
+        Returns
+        --------
+        :class:`Asset`
+            The resulting CDN asset.
+        """
+        return Asset._from_guild_avatar(self._state, self, format=format, static_format=static_format, size=size)
+
     async def ban(self, **kwargs):
         """|coro|
 
@@ -556,12 +615,12 @@ class Member(discord.abc.Messageable, _BaseUser):
         """
         await self.guild.ban(self, **kwargs)
 
-    async def unban(self, *, reason=None):
+    async def unban(self):
         """|coro|
 
         Unbans this member. Equivalent to :meth:`Guild.unban`.
         """
-        await self.guild.unban(self, reason=reason)
+        await self.guild.unban(self)
 
     async def kick(self, *, reason=None):
         """|coro|
@@ -570,7 +629,7 @@ class Member(discord.abc.Messageable, _BaseUser):
         """
         await self.guild.kick(self, reason=reason)
 
-    async def edit(self, *, reason=None, **fields):
+    async def edit(self, **fields):
         """|coro|
 
         Edits the member's data.
@@ -614,8 +673,6 @@ class Member(discord.abc.Messageable, _BaseUser):
         voice_channel: Optional[:class:`VoiceChannel`]
             The voice channel to move the member to.
             Pass ``None`` to kick them from voice.
-        reason: Optional[:class:`str`]
-            The reason for editing this member. Shows up on the audit log.
 
         Raises
         -------
@@ -637,7 +694,7 @@ class Member(discord.abc.Messageable, _BaseUser):
         else:
             nick = nick or ''
             if me:
-                await http.change_my_nickname(guild_id, nick, reason=reason)
+                await http.change_my_nickname(guild_id, nick)
             else:
                 payload['nick'] = nick
 
@@ -681,7 +738,7 @@ class Member(discord.abc.Messageable, _BaseUser):
             payload['roles'] = tuple(r.id for r in roles)
 
         if payload:
-            await http.edit_member(guild_id, self.id, reason=reason, **payload)
+            await http.edit_member(guild_id, self.id, **payload)
 
         # TODO: wait for WS event for modify-in-place behaviour
 
@@ -717,7 +774,7 @@ class Member(discord.abc.Messageable, _BaseUser):
         else:
             await self._state.http.edit_my_voice_state(self.guild.id, payload)
 
-    async def move_to(self, channel, *, reason=None):
+    async def move_to(self, channel):
         """|coro|
 
         Moves a member to a new voice channel (they must be connected first).
@@ -735,12 +792,10 @@ class Member(discord.abc.Messageable, _BaseUser):
         channel: Optional[:class:`VoiceChannel`]
             The new voice channel to move the member to.
             Pass ``None`` to kick them from voice.
-        reason: Optional[:class:`str`]
-            The reason for doing this action. Shows up on the audit log.
         """
-        await self.edit(voice_channel=channel, reason=reason)
+        await self.edit(voice_channel=channel)
 
-    async def add_roles(self, *roles, reason=None, atomic=True):
+    async def add_roles(self, *roles, atomic=True):
         r"""|coro|
 
         Gives the member a number of :class:`Role`\s.
@@ -754,8 +809,6 @@ class Member(discord.abc.Messageable, _BaseUser):
         \*roles: :class:`abc.Snowflake`
             An argument list of :class:`abc.Snowflake` representing a :class:`Role`
             to give to the member.
-        reason: Optional[:class:`str`]
-            The reason for adding these roles. Shows up on the audit log.
         atomic: :class:`bool`
             Whether to atomically add roles. This will ensure that multiple
             operations will always be applied regardless of the current
@@ -771,15 +824,15 @@ class Member(discord.abc.Messageable, _BaseUser):
 
         if not atomic:
             new_roles = utils._unique(Object(id=r.id) for s in (self.roles[1:], roles) for r in s)
-            await self.edit(roles=new_roles, reason=reason)
+            await self.edit(roles=new_roles)
         else:
             req = self._state.http.add_role
             guild_id = self.guild.id
             user_id = self.id
             for role in roles:
-                await req(guild_id, user_id, role.id, reason=reason)
+                await req(guild_id, user_id, role.id)
 
-    async def remove_roles(self, *roles, reason=None, atomic=True):
+    async def remove_roles(self, *roles, atomic=True):
         r"""|coro|
 
         Removes :class:`Role`\s from this member.
@@ -793,8 +846,6 @@ class Member(discord.abc.Messageable, _BaseUser):
         \*roles: :class:`abc.Snowflake`
             An argument list of :class:`abc.Snowflake` representing a :class:`Role`
             to remove from the member.
-        reason: Optional[:class:`str`]
-            The reason for removing these roles. Shows up on the audit log.
         atomic: :class:`bool`
             Whether to atomically remove roles. This will ensure that multiple
             operations will always be applied regardless of the current
@@ -816,10 +867,24 @@ class Member(discord.abc.Messageable, _BaseUser):
                 except ValueError:
                     pass
 
-            await self.edit(roles=new_roles, reason=reason)
+            await self.edit(roles=new_roles)
         else:
             req = self._state.http.remove_role
             guild_id = self.guild.id
             user_id = self.id
             for role in roles:
-                await req(guild_id, user_id, role.id, reason=reason)
+                await req(guild_id, user_id, role.id)
+
+    async def send_friend_request(self):
+        """|coro|
+
+        Sends the member a friend request.
+
+        Raises
+        -------
+        Forbidden
+            Not allowed to send a friend request to the member.
+        HTTPException
+            Sending the friend request failed.
+        """
+        await self._state.http.add_relationship(self._user.id, action=RelationshipAction.send_friend_request)
