@@ -28,18 +28,15 @@ import asyncio
 from collections import deque, OrderedDict
 import copy
 import datetime
-import itertools
 import logging
-import weakref
-import warnings
+from weakref import WeakValueDictionary
 import inspect
-import gc
 
 import os
 
 from .guild import Guild
 from .activity import BaseActivity
-from .user import User, ClientUser, LazyUser
+from .user import User, ClientUser
 from .emoji import Emoji
 from .mentions import AllowedMentions
 from .partial_emoji import PartialEmoji
@@ -49,11 +46,12 @@ from .channel import *
 from .raw_models import *
 from .member import Member
 from .role import Role
-from .enums import ChannelType, try_enum, Status
+from .enums import ChannelType, try_enum, Status, UnavailableGuildType, RequiredActionType, VoiceRegion
 from . import utils
-from .flags import Intents, MemberCacheFlags
+from .flags import GuildSubscriptionOptions, MemberCacheFlags
 from .object import Object
 from .invite import Invite
+from .settings import Settings
 
 class ChunkRequest:
     def __init__(self, guild_id, loop, resolver, *, cache=True):
@@ -104,7 +102,7 @@ async def logging_coroutine(coroutine, *, info):
         log.exception('Exception occurred during %s', info)
 
 class ConnectionState:
-    def __init__(self, *, dispatch, handlers, hooks, syncer, http, loop, **options):
+    def __init__(self, *, dispatch, handlers, hooks, http, loop, **options):
         self.loop = loop
         self.http = http
         self.max_messages = options.get('max_messages', 1000)
@@ -112,18 +110,11 @@ class ConnectionState:
             self.max_messages = 1000
 
         self.dispatch = dispatch
-        self.syncer = syncer
-        self.is_bot = None
         self.handlers = handlers
         self.hooks = hooks
-        self.shard_count = None
         self._ready_task = None
         self.heartbeat_timeout = options.get('heartbeat_timeout', 60.0)
-        self.guild_ready_timeout = options.get('guild_ready_timeout', 2.0)
-        if self.guild_ready_timeout < 0:
-            raise ValueError('guild_ready_timeout cannot be negative')
 
-        self.guild_subscriptions = options.get('guild_subscriptions', True)
         allowed_mentions = options.get('allowed_mentions')
 
         if allowed_mentions is not None and not isinstance(allowed_mentions, AllowedMentions):
@@ -141,51 +132,35 @@ class ConnectionState:
 
         status = options.get('status', None)
         if status:
+            if not isinstance(status, Status):
+                raise TypeError('status must be Status not {0.__class__!r}'.format(status))
             if status is Status.offline:
                 status = 'invisible'
             else:
                 status = str(status)
 
-        intents = options.get('intents', None)
-        if intents is not None:
-            if not isinstance(intents, Intents):
-                raise TypeError('intents parameter must be Intent not %r' % type(intents))
-        else:
-            intents = Intents.all()
-
-        if not intents.guilds:
-            log.warning('Guilds intent seems to be disabled. This may cause state related issues.')
-
-        try:
-            chunk_guilds = options['fetch_offline_members']
-        except KeyError:
-            chunk_guilds = options.get('chunk_guilds_at_startup', intents.members)
-        else:
-            msg = 'fetch_offline_members is deprecated, use chunk_guilds_at_startup instead'
-            warnings.warn(msg, DeprecationWarning, stacklevel=4)
-
+        chunk_guilds = options.get('chunk_guilds_at_startup', True)
         self._chunk_guilds = chunk_guilds
 
-        # Ensure these two are set properly
-        if not intents.members and self._chunk_guilds:
-            raise ValueError('Intents.members must be enabled to chunk guilds at startup.')
+        subscription_options = options.get('guild_subscription_options')
+        if subscription_options is None:
+            subscription_options = GuildSubscriptionOptions.default()
+        else:
+            if not isinstance(subscription_options, GuildSubscriptionOptions):
+                raise TypeError('subscription_options parameter must be GuildSubscriptionOptions not %r' % type(subscription_options))
+        self._subscription_options = subscription_options
+        self._subscribe_guilds = subscription_options.auto_subscribe
 
-        cache_flags = options.get('member_cache_flags', None)
+        cache_flags = options.get('member_cache_flags')
         if cache_flags is None:
-            cache_flags = MemberCacheFlags.from_intents(intents)
+            cache_flags = MemberCacheFlags.all()
         else:
             if not isinstance(cache_flags, MemberCacheFlags):
                 raise TypeError('member_cache_flags parameter must be MemberCacheFlags not %r' % type(cache_flags))
 
-            cache_flags._verify_intents(intents)
-
         self.member_cache_flags = cache_flags
         self._activity = activity
         self._status = status
-        self._intents = intents
-
-        if not intents.members or cache_flags._empty:
-            self.store_user = self.store_user_no_intents
 
         self.parsers = parsers = {}
         for attr, func in inspect.getmembers(self):
@@ -196,10 +171,12 @@ class ConnectionState:
 
     def clear(self):
         self.user = None
-        self._users = weakref.WeakValueDictionary()
+        self._users = WeakValueDictionary()
         self._emojis = {}
         self._calls = {}
         self._guilds = {}
+        self._unavailable_guilds = {}
+        self._queued_guilds = {}
         self._voice_clients = {}
 
         # LRU of max size 128
@@ -208,11 +185,6 @@ class ConnectionState:
         self._private_channels_by_user = {}
         self._messages = self.max_messages and deque(maxlen=self.max_messages)
 
-        # In cases of large deallocations the GC should be called explicitly
-        # To free the memory more immediately, especially true when it comes
-        # to reconnect loops which cause mass allocations and deallocations.
-        gc.collect()
-
     def process_chunk_requests(self, guild_id, nonce, members, complete):
         removed = []
         for key, request in self._chunk_requests.items():
@@ -246,12 +218,6 @@ class ConnectionState:
         u = self.user
         return u.id if u else None
 
-    @property
-    def intents(self):
-        ret = Intents.none()
-        ret.value = self._intents.value
-        return ret
-
     @property
     def voice_clients(self):
         return list(self._voice_clients.values())
@@ -270,31 +236,24 @@ class ConnectionState:
             vc.main_ws = ws
 
     def store_user(self, data):
-        # this way is 300% faster than `dict.setdefault`.
+        # This way is 300% faster than `dict.setdefault`.
         user_id = int(data['id'])
         try:
-            return self._users[user_id]
+            user = self._users[user_id]
+            # We use the data available to us since we
+            # might not have events for that user.
+            # However, the data may only have an ID.
+            try:
+                user._update(data)
+            except KeyError:
+                pass
+            return user
         except KeyError:
             user = User(state=self, data=data)
             if user.discriminator != '0000':
                 self._users[user_id] = user
             return user
 
-    def store_lazy_user(self, user_id):
-        # this way is 300% faster than `dict.setdefault`.
-        user_id = int(user_id)
-        try:
-            return self._users[user_id]
-        except KeyError:
-            user = LazyUser(state=self, user_id=user_id)
-            if user.discriminator != '0000':
-                self._users[user_id] = user
-            return user
-        
-
-    def store_user_no_intents(self, data):
-        return User(state=self, data=data)
-
     def get_user(self, id):
         return self._users.get(id)
 
@@ -308,7 +267,10 @@ class ConnectionState:
         return list(self._guilds.values())
 
     def _get_guild(self, guild_id):
-        return self._guilds.get(guild_id)
+        guild = self._guilds.get(guild_id)
+        if guild is None:
+            guild = self._queued_guilds.get(guild_id)
+        return guild
 
     def _add_guild(self, guild):
         self._guilds[guild.id] = guild
@@ -321,10 +283,6 @@ class ConnectionState:
 
         del guild
 
-        # Much like clear(), if we have a massive deallocation
-        # then it's better to explicitly call the GC
-        gc.collect()
-
     @property
     def emojis(self):
         return list(self._emojis.values())
@@ -352,11 +310,6 @@ class ConnectionState:
         channel_id = channel.id
         self._private_channels[channel_id] = channel
 
-        if self.is_bot and len(self._private_channels) > 128:
-            _, to_remove = self._private_channels.popitem(last=False)
-            if isinstance(to_remove, DMChannel):
-                self._private_channels_by_user.pop(to_remove.recipient.id, None)
-
         if isinstance(channel, DMChannel):
             self._private_channels_by_user[channel.recipient.id] = channel
 
@@ -373,14 +326,27 @@ class ConnectionState:
     def _get_message(self, msg_id):
         return utils.find(lambda m: m.id == msg_id, reversed(self._messages)) if self._messages else None
 
-    def _add_guild_from_data(self, guild):
-        guild = Guild(data=guild, state=self)
-        self._add_guild(guild)
-        return guild
+    def _add_guild_from_data(self, guild, *, from_ready=False):
+        guild_id = int(guild['id'])
+        unavailable = guild.get('unavailable', False)
+
+        if not unavailable:
+            guild = Guild(data=guild, state=self)
+            self._add_guild(guild)
+            return guild
+        else:
+            self._unavailable_guilds[guild_id] = UnavailableGuildType.existing if from_ready else UnavailableGuildType.joined
+            log.debug('Forcing GUILD_CREATE for unavailable guild %s' % guild_id)
+            asyncio.ensure_future(self.request_guild(guild_id), loop=self.loop)
 
     def _guild_needs_chunking(self, guild):
-        # If presences are enabled then we get back the old guild.large behaviour
-        return self._chunk_guilds and not guild.chunked and not (self._intents.presences and not guild.large)
+        try:
+            return self._chunk_guilds and not guild.chunked and guild.me.guild_permissions.manage_guild
+        except:
+            return False
+
+    def _guild_needs_subscribing(self, guild):
+        return not guild.subscribed and self._subscribe_guilds
 
     def _get_guild_channel(self, data):
         channel_id = int(data['channel_id'])
@@ -394,22 +360,23 @@ class ConnectionState:
 
         return channel or Object(id=channel_id), guild
 
-    async def chunker(self, guild_id, query='', limit=0, presences=False, *, nonce=None):
-        ws = self._get_websocket(guild_id) # This is ignored upstream
-        await ws.request_chunks(guild_id, query=query, limit=limit, presences=presences, nonce=nonce)
+    async def request_guild(self, guild_id):
+        ws = self._get_websocket()
+        await ws.request_lazy_guild(guild_id, typing=True, activities=True, threads=True)
+
+    async def chunker(self, guild_id, query='', limit=0, presences=True, *, nonce=None):
+        ws = self._get_websocket()
+        await ws.request_chunks([guild_id], query=query, limit=limit, presences=presences, nonce=nonce)
 
     async def query_members(self, guild, query, limit, user_ids, cache, presences):
         guild_id = guild.id
-        ws = self._get_websocket(guild_id)
-        if ws is None:
-            raise RuntimeError('Somehow do not have a websocket for this guild_id')
+        ws = self._get_websocket()
 
         request = ChunkRequest(guild.id, self.loop, self._get_guild, cache=cache)
         self._chunk_requests[request.nonce] = request
 
         try:
-            # start the query operation
-            await ws.request_chunks(guild_id, query=query, limit=limit, user_ids=user_ids, presences=presences, nonce=request.nonce)
+            await ws.request_chunks([guild_id], query=query, limit=limit, user_ids=user_ids, presences=presences, nonce=request.nonce)
             return await asyncio.wait_for(request.wait(), timeout=30.0)
         except asyncio.TimeoutError:
             log.warning('Timed out waiting for chunks with query %r and limit %d for guild_id %d', query, limit, guild_id)
@@ -417,81 +384,109 @@ class ConnectionState:
 
     async def _delay_ready(self):
         try:
-            # only real bots wait for GUILD_CREATE streaming
-            if self.is_bot:
-                states = []
-                while True:
-                    # this snippet of code is basically waiting N seconds
-                    # until the last GUILD_CREATE was sent
-                    try:
-                        guild = await asyncio.wait_for(self._ready_state.get(), timeout=self.guild_ready_timeout)
-                    except asyncio.TimeoutError:
-                        break
-                    else:
-                        if self._guild_needs_chunking(guild):
-                            future = await self.chunk_guild(guild, wait=False)
-                            states.append((guild, future))
-                        else:
-                            if guild.unavailable is False:
-                                self.dispatch('guild_available', guild)
-                            else:
-                                self.dispatch('guild_join', guild)
+            states = []
+            subscribes = []
+            for guild in self._guilds.values():
+                await self.request_guild(guild.id)
 
-                for guild, future in states:
-                    try:
-                        await asyncio.wait_for(future, timeout=5.0)
-                    except asyncio.TimeoutError:
-                        log.warning('Shard ID %s timed out waiting for chunks for guild_id %s.', guild.shard_id, guild.id)
+                if self._guild_needs_chunking(guild):
+                    future = await self.chunk_guild(guild, wait=False)
+                    states.append((guild, future))
 
-                    if guild.unavailable is False:
-                        self.dispatch('guild_available', guild)
-                    else:
-                        self.dispatch('guild_join', guild)
+                if self._guild_needs_subscribing(guild):
+                    subscribes.append(guild)
 
-            # remove the state
-            try:
-                del self._ready_state
-            except AttributeError:
-                pass # already been deleted somehow
+            for guild, future in states:
+                try:
+                    await asyncio.wait_for(future, timeout=5.0)
+                except asyncio.TimeoutError:
+                    log.warning('Bot timed out waiting for chunks for guild_id %s.', guild.id)
+
+            options = self._subscription_options
+            ticket = asyncio.Semaphore(options.concurrent_guilds)
+            await asyncio.gather(*[guild.subscribe(ticket=ticket, max_online=options.max_online) for guild in subscribes])
 
-            # call GUILD_SYNC after we're done chunking
-            if not self.is_bot:
-                log.info('Requesting GUILD_SYNC for %s guilds', len(self.guilds))
-                await self.syncer([s.id for s in self.guilds])
         except asyncio.CancelledError:
             pass
         else:
-            # dispatch the event
+            # Dispatch the event
             self.call_handlers('ready')
             self.dispatch('ready')
         finally:
             self._ready_task = None
 
     def parse_ready(self, data):
+        # Before parsing, we wait for READY_SUPPLEMENTAL.
+        # This has voice state objects, as well as a few other goodies.
+        self._ready_data = data
+
+    def parse_ready_supplemental(self, data):
         if self._ready_task is not None:
             self._ready_task.cancel()
 
-        self._ready_state = asyncio.Queue()
         self.clear()
+
+        # Merge with READY data
+        extra_data = data
+        data = self._ready_data
+
+        for guild_data, guild_extra, merged_members, merged_me, merged_presences in zip(
+            data.get('guilds', []),
+            extra_data.get('guilds', []),
+            extra_data.get('merged_members', []),
+            data.get('merged_members', []),
+            extra_data['merged_presences'].get('guilds', [])
+        ):
+            guild_data['voice_states'] = guild_extra.get('voice_states', [])
+            guild_data['merged_members'] = merged_me
+            guild_data['merged_members'].extend(merged_members)
+            guild_data['merged_presences'] = merged_presences
+            # There's also a friends key that has presence data for your friends.
+            # Parsing that would require a redesign of the Relationship class ;-;.
+
+        # Self parsing
         self.user = user = ClientUser(state=self, data=data['user'])
         self._users[user.id] = user
-        
-        for merged_member_list, guild_data in zip(data['merged_members'], data['guilds']):
-            guild_data['me'] = merged_member_list
-            self._add_guild_from_data(guild_data)
 
+        # Temp user parsing
+        temp_users = {user.id: user._to_minimal_user_json()}
+        for u in data.get('users', []):
+            u_id = int(u['id'])
+            temp_users[u_id] = u
+
+        # Guild parsing
+        for guild_data in data.get('guilds', []):
+            for member in guild_data['merged_members']:
+                if 'user' not in member:
+                    member['user'] = temp_users.get(int(member.pop('user_id')))
+            self._add_guild_from_data(guild_data, from_ready=True)
+
+        # Relationship parsing
         for relationship in data.get('relationships', []):
             try:
                 r_id = int(relationship['id'])
             except KeyError:
                 continue
             else:
+                if 'user' not in relationship:
+                    relationship['user'] = temp_users[int(relationship.pop('user_id'))]
                 user._relationships[r_id] = Relationship(state=self, data=relationship)
 
+        # Private channel parsing
         for pm in data.get('private_channels', []):
             factory, _ = _channel_factory(pm['type'])
+            if 'recipients' not in pm:
+                pm['recipients'] = [temp_users[int(u_id)] for u_id in pm.pop('recipient_ids')]
             self._add_private_channel(factory(me=user, data=pm, state=self))
 
+        # Extras
+        region = data.get('geo_ordered_rtc_regions', ['us-west'])[0]
+        self.preferred_region = try_enum(VoiceRegion, region)
+        user.settings = Settings(data=data.get('user_settings', {}), state=self)
+
+        # We're done
+        del self._ready_data
+        self.call_handlers('connect')
         self.dispatch('connect')
         self._ready_task = asyncio.ensure_future(self._delay_ready(), loop=self.loop)
 
@@ -499,6 +494,9 @@ class ConnectionState:
         self.dispatch('resumed')
 
     def parse_message_create(self, data):
+        guild_id = data.get('guild_id')
+        if guild_id is not None and int(guild_id) in self._unavailable_guilds:
+            return
         channel, _ = self._get_guild_channel(data)
         message = Message(channel=channel, data=data, state=self)
         self.dispatch('message', message)
@@ -615,6 +613,8 @@ class ConnectionState:
                     self.dispatch('reaction_clear_emoji', reaction)
 
     def parse_presence_update(self, data):
+        # Testing shows that this is only sent for relationship presences,
+        # which are not currently parsed.
         guild_id = utils._get_as_snowflake(data, 'guild_id')
         guild = self._get_guild(guild_id)
         if guild is None:
@@ -627,8 +627,6 @@ class ConnectionState:
         flags = self.member_cache_flags
         if member is None:
             if 'username' not in user:
-                # sometimes we receive 'incomplete' member data post-removal.
-                # skip these useless cases.
                 return
 
             member, old_member = Member._from_presence_update(guild=guild, data=data, state=self)
@@ -665,8 +663,8 @@ class ConnectionState:
                 guild._remove_channel(channel)
                 self.dispatch('guild_channel_delete', channel)
         else:
-            # the reason we're doing this is so it's also removed from the
-            # private channel by user cache as well
+            # The reason we're doing this is so it's also removed from the
+            # private channel by user cache as well.
             channel = self._get_private_channel(channel_id)
             if channel is not None:
                 self._remove_private_channel(channel)
@@ -728,9 +726,9 @@ class ConnectionState:
         last_pin = utils.parse_time(data['last_pin_timestamp']) if data['last_pin_timestamp'] else None
 
         try:
-            # I have not imported discord.abc in this file
-            # the isinstance check is also 2x slower than just checking this attribute
-            # so we're just gonna check it since it's easier and faster and lazier
+            # I have not imported discord.abc in this file;
+            # the isinstance check is also 2x slower than just checking this attribute,
+            # so we're just gonna check it since it's easier and faster and lazier.
             channel.guild
         except AttributeError:
             self.dispatch('private_channel_pins_update', channel, last_pin)
@@ -754,6 +752,9 @@ class ConnectionState:
             self.dispatch('group_remove', channel, user)
 
     def parse_guild_member_add(self, data):
+        # IIRC this is sent when you join a guild...
+        # This is currently useless since the guild is
+        # probably in the middle of being subscribed.
         guild = self._get_guild(int(data['guild_id']))
         if guild is None:
             log.debug('GUILD_MEMBER_ADD referencing an unknown guild ID: %s. Discarding.', data['guild_id'])
@@ -768,9 +769,11 @@ class ConnectionState:
         except AttributeError:
             pass
 
-        self.dispatch('member_join', member)
+        #self.dispatch('member_join', member)
 
     def parse_guild_member_remove(self, data):
+        # In rare events this is sent. Will probably be used more
+        # in the future (when on_member_remove returns).
         guild = self._get_guild(int(data['guild_id']))
         if guild is not None:
             try:
@@ -782,7 +785,7 @@ class ConnectionState:
             member = guild.get_member(user_id)
             if member is not None:
                 guild._remove_member(member)
-                self.dispatch('member_remove', member)
+                #self.dispatch('member_remove', member)
         else:
             log.debug('GUILD_MEMBER_REMOVE referencing an unknown guild ID: %s. Discarding.', data['guild_id'])
 
@@ -815,6 +818,100 @@ class ConnectionState:
                 guild._add_member(member)
             log.debug('GUILD_MEMBER_UPDATE referencing an unknown member ID: %s. Discarding.', user_id)
 
+    def parse_guild_sync(self, data):
+        print('HOW THE FUCK DID YOU TRIGGER A `GUILD_SYNC`???\nIf you want to share your secrets, please feel free to email me.')
+
+    def parse_guild_member_list_update(self, data):
+        self.dispatch('raw_guild_member_list_update', data)
+        guild = self._get_guild(int(data['guild_id']))
+        if guild is None:
+            log.debug('GUILD_MEMBER_LIST_UPDATE referencing an unknown guild ID: %s. Discarding.', data['guild_id'])
+            return
+
+        ops = data['ops']
+
+        if data['member_count'] > 0:
+            guild._member_count = data['member_count']
+
+        online_count = 0
+        for group in data['groups']:
+            online_count += group['count'] if group['id'] != 'offline' else 0
+        guild._online_count = online_count
+
+        for opdata in ops:
+            op = opdata['op']
+            # There are two OPs I'm not parsing.
+            # INVALIDATE: Usually invalid (hehe).
+            # DELETE: Sends the index, not the user ID, so I can't do anything with
+            # it unless I keep a seperate list of the member sidebar (maybe in future).
+
+            if op == 'SYNC':
+                members = [Member(guild=guild, data=member['member'], state=self) for member in [item for item in opdata.get('items', []) if 'member' in item]]
+
+                member_dict = {str(member.id): member for member in members}
+                for presence in [item for item in opdata.get('items', []) if 'member' in item]:
+                    presence = presence['member']['presence']
+                    user = presence['user']
+                    member_id = user['id']
+                    member = member_dict.get(member_id)
+                    member._presence_update(presence, user)
+
+                for member in members:
+                    guild._add_member(member)
+
+            if op == 'INSERT':
+                if 'member' not in opdata['item']:
+                    # Hoisted role INSERT
+                    return
+
+                mdata = opdata['item']['member']
+                user = mdata['user']
+                user_id = int(user['id'])
+
+                member = guild.get_member(user_id)
+                if member is not None: # INSERTs are also sent when a user changes range
+                    old_member = Member._copy(member)
+                    member._update(mdata)
+                    user_update = member._update_inner_user(user)
+                    if 'presence' in mdata:
+                        presence = mdata['presence']
+                        user = presence['user']
+                        member_id = user['id']
+                        member._presence_update(presence, user)
+                    if user_update:
+                        self.dispatch('user_update', user_update[0], user_update[1])
+
+                    self.dispatch('member_update', old_member, member)
+                else:
+                    member = Member(data=mdata, guild=guild, state=self)
+                    guild._add_member(member)
+
+            if op == 'UPDATE':
+                if 'member' not in opdata['item']:
+                    # Hoisted role UPDATE
+                    return
+
+                mdata = opdata['item']['member']
+                user = mdata['user']
+                user_id = int(user['id'])
+
+                member = guild.get_member(user_id)
+                if member is not None:
+                    old_member = Member._copy(member)
+                    member._update(mdata)
+                    user_update = member._update_inner_user(user)
+                    if 'presence' in mdata:
+                        presence = mdata['presence']
+                        user = presence['user']
+                        member_id = user['id']
+                        member._presence_update(presence, user)
+                    if user_update:
+                        self.dispatch('user_update', user_update[0], user_update[1])
+
+                    self.dispatch('member_update', old_member, member)
+                else:
+                    log.debug('GUILD_MEMBER_LIST_UPDATE type UPDATE referencing an unknown member ID: %s. Discarding.', user_id)
+
     def parse_guild_emojis_update(self, data):
         guild = self._get_guild(int(data['guild_id']))
         if guild is None:
@@ -828,15 +925,14 @@ class ConnectionState:
         self.dispatch('guild_emojis_update', guild, before_emojis, guild.emojis)
 
     def _get_create_guild(self, data):
-        if data.get('unavailable') is False:
-            # GUILD_CREATE with unavailable in the response
-            # usually means that the guild has become available
-            # and is therefore in the cache
-            guild = self._get_guild(int(data['id']))
-            if guild is not None:
-                guild.unavailable = False
-                guild._from_data(data)
-                return guild
+        guild = self._get_guild(int(data['id']))
+        # Discord being Discord sends a GUILD_CREATE
+        # after an OPCode 14 is sent (a la bots).
+        # However, we want that if we forced a
+        # GUILD_CREATE for an unavailable guild.
+        if guild is not None:
+            guild._from_data(data)
+            return
 
         return self._add_guild_from_data(data)
 
@@ -854,49 +950,54 @@ class ConnectionState:
             return await request.wait()
         return request.get_future()
 
-    async def _chunk_and_dispatch(self, guild, unavailable):
-        try:
-            await asyncio.wait_for(self.chunk_guild(guild), timeout=60.0)
-        except asyncio.TimeoutError:
-            log.info('Somehow timed out waiting for chunks.')
+    async def _parse_and_dispatch(self, guild, *, chunk, subscribe):
+        self._queued_guilds[guild.id] = guild
 
-        if unavailable is False:
-            self.dispatch('guild_available', guild)
+        if chunk:
+            try:
+                await asyncio.wait_for(self.chunk_guild(guild), timeout=60.0)
+            except asyncio.TimeoutError:
+                log.info('Somehow timed out waiting for chunks.')
+
+        if subscribe:
+            await guild.subscribe(max_online=self._subscription_options.max_online)
+
+        self._queued_guilds.pop(guild.id)
+
+        # Dispatch available/join depending on circumstances
+        if guild.id in self._unavailable_guilds:
+            type = self._unavailable_guilds.pop(guild.id)
+            if type is UnavailableGuildType.existing:
+                self.dispatch('guild_available', guild)
+            else:
+                self.dispatch('guild_join', guild)
         else:
             self.dispatch('guild_join', guild)
 
     def parse_guild_create(self, data):
-        unavailable = data.get('unavailable')
-        if unavailable is True:
-            # joined a guild with unavailable == True so..
-            return
+        guild_id = int(data['id'])
 
         guild = self._get_create_guild(data)
 
-        try:
-            # Notify the on_ready state, if any, that this guild is complete.
-            self._ready_state.put_nowait(guild)
-        except AttributeError:
-            pass
-        else:
-            # If we're waiting for the event, put the rest on hold
+        if guild is None:
             return
 
-        # check if it requires chunking
-        if self._guild_needs_chunking(guild):
-            asyncio.ensure_future(self._chunk_and_dispatch(guild, unavailable), loop=self.loop)
+        # Chunk/subscribe if needed
+        needs_chunking, needs_subscribing = self._guild_needs_chunking(guild), self._guild_needs_subscribing(guild)
+        if needs_chunking or needs_subscribing:
+            asyncio.ensure_future(self._parse_and_dispatch(guild, chunk=needs_chunking, subscribe=needs_subscribing), loop=self.loop)
             return
 
-        # Dispatch available if newly available
-        if unavailable is False:
-            self.dispatch('guild_available', guild)
+        # Dispatch available/join depending on circumstances
+        if guild_id in self._unavailable_guilds:
+            type = self._unavailable_guilds.pop(guild_id)
+            if type is UnavailableGuildType.existing:
+                self.dispatch('guild_available', guild)
+            else:
+                self.dispatch('guild_join', guild)
         else:
             self.dispatch('guild_join', guild)
 
-    def parse_guild_sync(self, data):
-        guild = self._get_guild(int(data['id']))
-        guild._sync(data)
-
     def parse_guild_update(self, data):
         guild = self._get_guild(int(data['id']))
         if guild is not None:
@@ -919,7 +1020,7 @@ class ConnectionState:
             self.dispatch('guild_unavailable', guild)
             return
 
-        # do a cleanup of the messages cache
+        # Cleanup the message cache
         if self._messages is not None:
             self._messages = deque((msg for msg in self._messages if msg.guild != guild), maxlen=self.max_messages)
 
@@ -927,11 +1028,6 @@ class ConnectionState:
         self.dispatch('guild_remove', guild)
 
     def parse_guild_ban_add(self, data):
-        # we make the assumption that GUILD_BAN_ADD is done
-        # before GUILD_MEMBER_REMOVE is called
-        # hence we don't remove it from cache or do anything
-        # strange with it, the main purpose of this event
-        # is mainly to dispatch to another event worth listening to for logging
         guild = self._get_guild(int(data['guild_id']))
         if guild is not None:
             try:
@@ -1034,7 +1130,7 @@ class ConnectionState:
             if member is not None:
                 if flags.voice:
                     if channel_id is None and flags._voice_only and member.id != self_id:
-                        # Only remove from cache iff we only have the voice flag enabled
+                        # Only remove from cache if we only have the voice flag enabled
                         guild._remove_member(member)
                     elif channel_id is not None:
                         guild._add_member(member)
@@ -1043,7 +1139,7 @@ class ConnectionState:
             else:
                 log.debug('VOICE_STATE_UPDATE referencing an unknown member ID: %s. Discarding.', data['user_id'])
         else:
-            # in here we're either at private or group calls
+            # We're at calls
             call = self._calls.get(channel_id)
             if call is not None:
                 call._update_voice_state(data)
@@ -1099,6 +1195,10 @@ class ConnectionState:
         else:
             self.dispatch('relationship_remove', old)
 
+    def parse_user_required_action_update(self, data):
+        required_action = try_enum(RequiredActionType, data['required_action'])
+        self.dispatch('required_action_update', required_action)
+
     def _get_reaction_user(self, channel, user_id):
         if isinstance(channel, TextChannel):
             return channel.guild.get_member(user_id)
@@ -1139,127 +1239,3 @@ class ConnectionState:
 
     def create_message(self, *, channel, data):
         return Message(state=self, channel=channel, data=data)
-
-class AutoShardedConnectionState(ConnectionState):
-    def __init__(self, *args, **kwargs):
-        super().__init__(*args, **kwargs)
-        self._ready_task = None
-        self.shard_ids = ()
-        self.shards_launched = asyncio.Event()
-
-    def _update_message_references(self):
-        for msg in self._messages:
-            if not msg.guild:
-                continue
-
-            new_guild = self._get_guild(msg.guild.id)
-            if new_guild is not None and new_guild is not msg.guild:
-                channel_id = msg.channel.id
-                channel = new_guild.get_channel(channel_id) or Object(id=channel_id)
-                msg._rebind_channel_reference(channel)
-
-    async def chunker(self, guild_id, query='', limit=0, presences=False, *, shard_id=None, nonce=None):
-        ws = self._get_websocket(guild_id, shard_id=shard_id)
-        await ws.request_chunks(guild_id, query=query, limit=limit, presences=presences, nonce=nonce)
-
-    async def _delay_ready(self):
-        await self.shards_launched.wait()
-        processed = []
-        max_concurrency = len(self.shard_ids) * 2
-        current_bucket = []
-        while True:
-            # this snippet of code is basically waiting N seconds
-            # until the last GUILD_CREATE was sent
-            try:
-                guild = await asyncio.wait_for(self._ready_state.get(), timeout=self.guild_ready_timeout)
-            except asyncio.TimeoutError:
-                break
-            else:
-                if self._guild_needs_chunking(guild):
-                    log.debug('Guild ID %d requires chunking, will be done in the background.', guild.id)
-                    if len(current_bucket) >= max_concurrency:
-                        try:
-                            await utils.sane_wait_for(current_bucket, timeout=max_concurrency * 70.0)
-                        except asyncio.TimeoutError:
-                            fmt = 'Shard ID %s failed to wait for chunks from a sub-bucket with length %d'
-                            log.warning(fmt, guild.shard_id, len(current_bucket))
-                        finally:
-                            current_bucket = []
-
-                    # Chunk the guild in the background while we wait for GUILD_CREATE streaming
-                    future = asyncio.ensure_future(self.chunk_guild(guild))
-                    current_bucket.append(future)
-                else:
-                    future = self.loop.create_future()
-                    future.set_result([])
-
-                processed.append((guild, future))
-
-        guilds = sorted(processed, key=lambda g: g[0].shard_id)
-        for shard_id, info in itertools.groupby(guilds, key=lambda g: g[0].shard_id):
-            children, futures = zip(*info)
-            # 110 reqs/minute w/ 1 req/guild plus some buffer
-            timeout = 61 * (len(children) / 110)
-            try:
-                await utils.sane_wait_for(futures, timeout=timeout)
-            except asyncio.TimeoutError:
-                log.warning('Shard ID %s failed to wait for chunks (timeout=%.2f) for %d guilds', shard_id,
-                                                                                                  timeout,
-                                                                                                  len(guilds))
-            for guild in children:
-                if guild.unavailable is False:
-                    self.dispatch('guild_available', guild)
-                else:
-                    self.dispatch('guild_join', guild)
-
-            self.dispatch('shard_ready', shard_id)
-
-        # remove the state
-        try:
-            del self._ready_state
-        except AttributeError:
-            pass # already been deleted somehow
-
-        # regular users cannot shard so we won't worry about it here.
-
-        # clear the current task
-        self._ready_task = None
-
-        # dispatch the event
-        self.call_handlers('ready')
-        self.dispatch('ready')
-
-    def parse_ready(self, data):
-        if not hasattr(self, '_ready_state'):
-            self._ready_state = asyncio.Queue()
-
-        self.user = user = ClientUser(state=self, data=data['user'])
-        self._users[user.id] = user
-
-        for guild_data in data['guilds']:
-            self._add_guild_from_data(guild_data)
-
-        if self._messages:
-            self._update_message_references()
-
-        for pm in data.get('private_channels', []):
-            factory, _ = _channel_factory(pm['type'])
-            self._add_private_channel(factory(me=user, data=pm, state=self))
-
-        self.dispatch('connect')
-        self.dispatch('shard_connect', data['__shard_id__'])
-
-        # Much like clear(), if we have a massive deallocation
-        # then it's better to explicitly call the GC
-        # Note that in the original ready parsing code this was done
-        # implicitly via clear() but in the auto sharded client clearing
-        # the cache would have the consequence of clearing data on other
-        # shards as well.
-        gc.collect()
-
-        if self._ready_task is None:
-            self._ready_task = asyncio.ensure_future(self._delay_ready(), loop=self.loop)
-
-    def parse_resumed(self, data):
-        self.dispatch('resumed')
-        self.dispatch('shard_resumed', data['__shard_id__'])
